export(normalize)
import(RCSF)
importFrom(Rcpp, evalCpp)
importFrom(data.table, ":=")
importFrom(tidyr, "%>%")
importFrom("stats", "sd", "uniroot", "var")
importFrom("graphics", "legend", "points")
importFrom("stats", "ave", "coef", "lm", "predict")
importFrom("utils", "read.csv")
useDynLib(FORTLS, .registration=TRUE)
exportPattern("^[[:alpha:]]+")
