# Info about Licenses

All the contents of this directory (original URL: <https://github.com/fortran-lang/fortls/tree/master/assets>) and any subsequent directories or links or copies of this directory
are licensed under the
[Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International](https://creativecommons.org/licenses/by-nc-nd/4.0/) license.

The `symbol-class.svg` is licensed by Microsoft under a [Creative Commons Attribution 4.0 International License](https://creativecommons.org/licenses/by/4.0/legalcode),
which allows for derivative works to change the original license.

The owner of this repository reserve all other rights, whether under their respective copyrights, patents, or trademarks, whether by implication, estoppel or otherwise.
