subroutine parse_line_continuations
  call report_test("[adaptivity output]", .false., .false., "Congratulations! &
                   & The output from adaptivity might even be OK if you get this far.")
end subroutine parse_line_continuations
