# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

geometric_features_dist <- function(points, dist, Anisotropy = FALSE, Eigenentropy = FALSE, Eigenvalue_sum = FALSE, First_eigenvalue = FALSE, Linearity = FALSE, Normal_x = FALSE, Normal_y = FALSE, Normal_z = FALSE, Number_of_points = FALSE, Omnivariance = FALSE, PCA_1 = FALSE, PCA_2 = FALSE, Planarity = FALSE, Second_eigenvalue = FALSE, Sphericity = FALSE, Surface_density = FALSE, Surface_variation = FALSE, Third_eigenvalue = FALSE, Verticality = FALSE, Volume_density = FALSE, num_threads = 0L, use_spatial_index = TRUE, grid_cell_size = -1.0) {
    .Call(`_FORTLS_geometric_features_dist`, points, dist, Anisotropy, Eigenentropy, Eigenvalue_sum, First_eigenvalue, Linearity, Normal_x, Normal_y, Normal_z, Number_of_points, Omnivariance, PCA_1, PCA_2, Planarity, Second_eigenvalue, Sphericity, Surface_density, Surface_variation, Third_eigenvalue, Verticality, Volume_density, num_threads, use_spatial_index, grid_cell_size)
}

fit_circle_cpp_modified <- function(points) {
    .Call(`_FORTLS_fit_circle_cpp_modified`, points)
}

is_one_row_all_na <- function(df) {
    .Call(`_FORTLS_is_one_row_all_na`, df)
}

sample_indices <- function(total_number_obs, return_obs) {
    .Call(`_FORTLS_sample_indices`, total_number_obs, return_obs)
}

internal_ransac <- function(dat, dist = 0.05) {
    .Call(`_FORTLS_internal_ransac`, dat, dist)
}

RANSAC_cpp <- function(data, dist = 0.05) {
    .Call(`_FORTLS_RANSAC_cpp`, data, dist)
}

iterations_RANSAC <- function(data, n_iterations = 600L) {
    .Call(`_FORTLS_iterations_RANSAC`, data, n_iterations)
}

weighted_mean_arit <- function(x, w) {
    .Call(`_FORTLS_weighted_mean_arit`, x, w)
}

weighted_mean_sqrt <- function(x, w) {
    .Call(`_FORTLS_weighted_mean_sqrt`, x, w)
}

weighted_mean_geom <- function(x, w) {
    .Call(`_FORTLS_weighted_mean_geom`, x, w)
}

weighted_mean_harm <- function(x, w) {
    .Call(`_FORTLS_weighted_mean_harm`, x, w)
}

height_perc_cpp <- function(rho_seq, z, rho) {
    .Call(`_FORTLS_height_perc_cpp`, rho_seq, z, rho)
}

fixed_area_cpp <- function(radius_seq, hdist, d, h, num) {
    .Call(`_FORTLS_fixed_area_cpp`, radius_seq, hdist, d, h, num)
}

k_tree_cpp <- function(k_seq, radius_seq, k, d, h, num) {
    .Call(`_FORTLS_k_tree_cpp`, k_seq, radius_seq, k, d, h, num)
}

angle_count_cpp <- function(baf_seq, baf, d, h, num) {
    .Call(`_FORTLS_angle_count_cpp`, baf_seq, baf, d, h, num)
}

