      program comment_as_reference
C     Comment with variable name gets picked as a ref: variable_to_reference
      real variable_to_reference
      variable_to_reference = 1
      end program comment_as_reference
