# Security Policy

## Supported Versions

`fortls` supports **ONLY** the latest Release. An autoupdate function is enabled by default to fetch the newest updates from `PyPi`.
For Anaconda environments the autoupdate functionality is disabled and it is up to the user to update to the latest version.


## Reporting a Vulnerability

The codebase is regularly scanned and patched for any potential security vulnerabilities.
If you manage to find a vulnerability in the Language Server please open an [Bug Report](https://github.com/fortran-lang/fortls/issues) with prefix: **SECURITY:**.
